#!/usr/bin/perl
#    Copyright 2010, 2011, Henning Kopp, Nina Bindel
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by   
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#     
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#     
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use Cwd;
use warnings;
use utf8;
use threads;
use IO::Handle;
use File::Basename;
no warnings 'threads';

#include  <qtcolorpicker.h>; 
#include <QHBoxLayout>;

package GUI;
use Qt;
use Qt::isa qw(Qt::MainWindow);
use Qt::slots
helpManual => [],
helpContents => [],
helpAbout => [],
addSurface => [],
selectAllDraw => [],
selectAllIntersect => [],
clean => [],
setStable => [],
verbosechanged => [],
draw => [],
sameState => [],
changeDraw => [],
transall => [],
changeIntersection => [],
colorText=>[],
changeColor => [];
use Qt::attributes qw(
colerintersectLabel
button0              
intersectLabel      
drawLabel
drawallBox        
surfaceLabel
stableCheckBox               
intersectallBox      
DrawButton        
MenuBar
fileMenu     
helpMenu    
fileExitAction  
helpManualAction
helpAboutAction  
fileClearAction  
);

#  button0          #Button to choose the color of the intersection
#  drawallBox        #Check Box to select all or none surface  to draw
#  stableCheckBox      #Check Box for Enable or disable the stable option   
#  intersectallBox      #Check Box to select all or none surface to interesect
#  DrawButton        #Click Button to start drawing
#  fileMenu        #in the File menu are the actions: fileClearAction and the fileExitAction 
#  helpMenu        #in the Help menu are the actions: helpManualAction, helpAboutAction
#  fileExitAction      #exit the program
#  helpManualAction    #opens the manual
#  helpAboutAction      #gives a short describtion of the software TropicalInsect und the GUI
#  fileClearAction      #delete all data in the gui, like you just start the program

###################some stuff puic created #############################
our $image0;
our $image0_data = pack 'C*',
0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
0x49, 0x48, 0x44, 0x52, 0x00, 0x00, 0x00, 0x16, 0x00, 0x00, 0x00, 0x16,
0x08, 0x06, 0x00, 0x00, 0x00, 0xc4, 0xb4, 0x6c, 0x3b, 0x00, 0x00, 0x00,
0x73, 0x49, 0x44, 0x41, 0x54, 0x38, 0x8d, 0xed, 0x92, 0xc1, 0x0e, 0x80,
0x20, 0x08, 0x40, 0xb1, 0xef, 0x66, 0x1d, 0x1c, 0xff, 0x4d, 0x87, 0x6a,
0xa8, 0x88, 0xa1, 0x76, 0x69, 0xf9, 0x36, 0x0f, 0x28, 0x3e, 0xd8, 0x00,
0x60, 0xf1, 0x59, 0x42, 0x5f, 0x3a, 0x71, 0xf5, 0x36, 0x02, 0xe0, 0x8e,
0x99, 0x2b, 0x09, 0x88, 0x01, 0xd0, 0x28, 0x54, 0x17, 0x6a, 0xe4, 0x7f,
0x21, 0xce, 0x1f, 0xb5, 0xb0, 0x5d, 0x38, 0xed, 0xdc, 0x90, 0x60, 0xd0,
0xf1, 0x13, 0x79, 0x63, 0x5b, 0x3b, 0xc9, 0x2b, 0xd5, 0x18, 0xe2, 0x39,
0xa9, 0x43, 0xec, 0x1d, 0x5a, 0xb7, 0x78, 0x5c, 0xee, 0x10, 0x7b, 0xe4,
0xb2, 0x15, 0xaf, 0x40, 0x91, 0xf8, 0x94, 0xde, 0x47, 0x18, 0x1e, 0xce,
0xa5, 0x9e, 0xde, 0x9e, 0xc5, 0x9f, 0x38, 0x00, 0x62, 0xac, 0x28, 0xb1,
0xe3, 0xd7, 0x01, 0xd9, 0x00, 0x00, 0x00, 0x00, 0x49, 0x45, 0x4e, 0x44,
0xae, 0x42, 0x60, 0x82;

our $image1;
our $image1_data = pack 'C*',
0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
0x49, 0x48, 0x44, 0x52, 0x00, 0x00, 0x00, 0x16, 0x00, 0x00, 0x00, 0x16,
0x08, 0x06, 0x00, 0x00, 0x00, 0xc4, 0xb4, 0x6c, 0x3b, 0x00, 0x00, 0x00,
0x74, 0x49, 0x44, 0x41, 0x54, 0x38, 0x8d, 0xed, 0x92, 0xc1, 0x0a, 0xc0,
0x20, 0x08, 0x40, 0x6d, 0xdf, 0x2d, 0x3b, 0x84, 0xff, 0xed, 0x0e, 0xa3,
0x58, 0x6a, 0x26, 0xd1, 0x65, 0xe0, 0x83, 0x0e, 0xa5, 0x3e, 0x85, 0x04,
0x48, 0x7e, 0x4b, 0x91, 0x0f, 0x54, 0x89, 0xf1, 0x9e, 0xa5, 0xa3, 0xca,
0x0f, 0x8a, 0x89, 0x63, 0x65, 0xb3, 0x06, 0xc4, 0x2d, 0xd6, 0x13, 0xc6,
0x49, 0xbd, 0xc2, 0x59, 0x83, 0x16, 0x13, 0x62, 0x19, 0xf0, 0xf9, 0x36,
0xc0, 0xa2, 0xef, 0x00, 0xd7, 0x5a, 0x62, 0x61, 0x4d, 0x3a, 0xb2, 0x29,
0x96, 0xf2, 0xa3, 0x62, 0xff, 0xa3, 0x37, 0xc5, 0xeb, 0xed, 0xe9, 0x62,
0xaa, 0xd1, 0xa2, 0xe8, 0x4a, 0xaa, 0xa2, 0xf7, 0x50, 0xdd, 0x12, 0x74,
0x8c, 0x0f, 0xd0, 0xab, 0x93, 0x24, 0x67, 0x78, 0x00, 0x59, 0x6e, 0x28,
0xb1, 0x74, 0x3f, 0x46, 0x86, 0x00, 0x00, 0x00, 0x00, 0x49, 0x45, 0x4e,
0x44, 0xae, 0x42, 0x60, 0x82;

our $image2;
our $image2_data = pack 'C*',
0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
0x49, 0x48, 0x44, 0x52, 0x00, 0x00, 0x00, 0x16, 0x00, 0x00, 0x00, 0x16,
0x08, 0x06, 0x00, 0x00, 0x00, 0xc4, 0xb4, 0x6c, 0x3b, 0x00, 0x00, 0x00,
0x82, 0x49, 0x44, 0x41, 0x54, 0x38, 0x8d, 0xcd, 0xd3, 0x41, 0x12, 0x80,
0x20, 0x08, 0x05, 0x50, 0xe8, 0xe0, 0x2e, 0xbc, 0x38, 0xad, 0x32, 0x73,
0x50, 0x3e, 0x48, 0x53, 0x7f, 0xe3, 0xe4, 0x8c, 0x4f, 0x24, 0x25, 0xfa,
0x28, 0xe2, 0x9c, 0x6f, 0x39, 0x92, 0x0b, 0xf9, 0x27, 0x6c, 0xb6, 0x01,
0x85, 0x35, 0x88, 0x77, 0x61, 0x13, 0x88, 0xc2, 0x57, 0x64, 0x18, 0xcd,
0xa0, 0x15, 0xf5, 0x20, 0xb4, 0xe6, 0xb5, 0x5b, 0xe1, 0x09, 0xdc, 0x06,
0x22, 0xb8, 0xe2, 0x2a, 0xcf, 0x31, 0x05, 0x6e, 0x18, 0xdf, 0xdf, 0xf8,
0x06, 0x06, 0xaa, 0x55, 0x1c, 0xc6, 0x35, 0x64, 0xc4, 0xdc, 0xf8, 0x0c,
0xd0, 0x20, 0x1d, 0x57, 0x7a, 0x5c, 0x85, 0xa8, 0x84, 0x5f, 0xdc, 0x02,
0x5e, 0xa5, 0x30, 0x7a, 0xfc, 0xcd, 0x07, 0xe2, 0x3a, 0x1d, 0xf2, 0x83,
0xec, 0x2b, 0x37, 0xd9, 0xad, 0x5f, 0xb4, 0xdf, 0xef, 0xd4, 0x9c, 0xfb,
0xf7, 0x2f, 0xac, 0x98, 0xc8, 0xcc, 0x89, 0x00, 0x00, 0x00, 0x00, 0x49,
0x45, 0x4e, 0x44, 0xae, 0x42, 0x60, 0x82;

our $image3;
our $image3_data = pack 'C*',
0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
0x49, 0x48, 0x44, 0x52, 0x00, 0x00, 0x00, 0x16, 0x00, 0x00, 0x00, 0x16,
0x08, 0x06, 0x00, 0x00, 0x00, 0xc4, 0xb4, 0x6c, 0x3b, 0x00, 0x00, 0x00,
0xbf, 0x49, 0x44, 0x41, 0x54, 0x38, 0x8d, 0xd5, 0x93, 0x41, 0x0a, 0x83,
0x30, 0x10, 0x45, 0xdf, 0x48, 0x8e, 0xe5, 0x1d, 0xbc, 0x8c, 0x3b, 0xa9,
0x8b, 0xf4, 0x6a, 0x9e, 0xab, 0xd3, 0x85, 0x35, 0x0d, 0x26, 0x63, 0x62,
0x44, 0x4a, 0x3f, 0x0c, 0x42, 0x66, 0xfc, 0xf3, 0xf8, 0x24, 0xf0, 0x6f,
0x12, 0x40, 0x2b, 0x66, 0xda, 0x8c, 0x55, 0xf3, 0xde, 0x22, 0x12, 0xcf,
0x9d, 0x92, 0xcb, 0x98, 0xc0, 0xba, 0x2d, 0x7c, 0x45, 0x44, 0xcf, 0x9a,
0x07, 0x63, 0x8b, 0xba, 0xd5, 0x3c, 0x44, 0x91, 0x23, 0x5e, 0xcf, 0x7c,
0xc1, 0x62, 0x36, 0x97, 0xa9, 0x25, 0x40, 0xc1, 0x1f, 0xf4, 0xfd, 0xa7,
0x52, 0x75, 0x01, 0x5d, 0x24, 0xa9, 0x38, 0x9e, 0x7d, 0x6f, 0x53, 0xdf,
0x4f, 0xe4, 0xcc, 0xab, 0x32, 0x3e, 0xea, 0x0f, 0x03, 0xc0, 0xc4, 0xb2,
0xa0, 0x71, 0x2c, 0xe6, 0xad, 0xd8, 0x9b, 0x59, 0xb7, 0x66, 0x1c, 0x3b,
0xe0, 0x95, 0x98, 0x5f, 0x26, 0x16, 0x79, 0xee, 0x4e, 0xbc, 0xc2, 0x2c,
0x97, 0x88, 0x55, 0x1f, 0xe6, 0xa2, 0xcb, 0xc4, 0x96, 0x9a, 0x89, 0x4b,
0xcb, 0x6f, 0x23, 0xee, 0x36, 0x1a, 0xab, 0x62, 0xe2, 0x52, 0xc5, 0x72,
0x94, 0xdf, 0xbf, 0xb6, 0x10, 0xbb, 0xf2, 0xc8, 0x97, 0xb8, 0xa4, 0x6c,
0xc6, 0x67, 0x7e, 0xaa, 0x51, 0x95, 0x71, 0xfa, 0x08, 0x7e, 0xa8, 0x37,
0x62, 0xda, 0x9a, 0xba, 0xcb, 0x20, 0x23, 0x5f, 0x00, 0x00, 0x00, 0x00,
0x49, 0x45, 0x4e, 0x44, 0xae, 0x42, 0x60, 0x82;

our $image4;
our $image4_data = pack 'C*',
0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
0x49, 0x48, 0x44, 0x52, 0x00, 0x00, 0x00, 0x16, 0x00, 0x00, 0x00, 0x16,
0x08, 0x06, 0x00, 0x00, 0x00, 0xc4, 0xb4, 0x6c, 0x3b, 0x00, 0x00, 0x00,
0xd5, 0x49, 0x44, 0x41, 0x54, 0x38, 0x8d, 0xc5, 0x95, 0x5d, 0x0a, 0x84,
0x30, 0x0c, 0x84, 0x27, 0xe2, 0xa9, 0x0a, 0x9e, 0x6c, 0x8b, 0x0f, 0x4b,
0x3d, 0xd9, 0x82, 0xd7, 0xca, 0x3e, 0x58, 0xd7, 0xfe, 0x4c, 0xd0, 0xba,
0x5d, 0x76, 0x40, 0x02, 0x4d, 0xf2, 0x65, 0xda, 0x0a, 0x05, 0x7e, 0x24,
0x39, 0xc9, 0xeb, 0x8d, 0x9e, 0xaa, 0x88, 0x41, 0xa0, 0xc9, 0xaa, 0xd8,
0xc8, 0x2a, 0xb3, 0x2f, 0x9c, 0x42, 0x5b, 0xe1, 0xe3, 0x0e, 0x0d, 0xcf,
0x00, 0xc0, 0x03, 0x08, 0xf0, 0xb3, 0xa7, 0xa0, 0x74, 0x10, 0xa9, 0xd7,
0x14, 0x2e, 0x00, 0xb4, 0x2c, 0x5a, 0x5f, 0xab, 0x69, 0x6b, 0x97, 0x9b,
0x1c, 0x83, 0x7f, 0xc0, 0xc3, 0x16, 0xb6, 0xe4, 0x16, 0x5b, 0x64, 0xf7,
0x8d, 0x71, 0x63, 0x59, 0x91, 0x9b, 0xdc, 0x45, 0x70, 0xde, 0x47, 0xc0,
0x47, 0x32, 0xdd, 0x5e, 0x5b, 0xcc, 0x35, 0xf0, 0xc9, 0x77, 0x62, 0xae,
0x78, 0x79, 0x36, 0xdc, 0xcf, 0x75, 0x13, 0x57, 0x7e, 0x79, 0xf4, 0x8c,
0x4b, 0x27, 0xaa, 0x0f, 0x13, 0x27, 0xb2, 0x40, 0xf5, 0x11, 0x7f, 0xcb,
0xe3, 0x48, 0xaa, 0x33, 0xb6, 0xe0, 0x22, 0x4b, 0x05, 0x4d, 0x07, 0x46,
0xb8, 0x02, 0x5e, 0x2e, 0x3b, 0x3e, 0x73, 0xcd, 0xe0, 0xdd, 0x1c, 0x97,
0xf0, 0x2e, 0x8e, 0xd9, 0xd0, 0xaf, 0x1d, 0xb3, 0x81, 0x22, 0x4b, 0xdf,
0x33, 0xee, 0xe6, 0x98, 0xa9, 0x34, 0xa0, 0xf6, 0x17, 0xb4, 0x55, 0x40,
0xd0, 0x0b, 0xcf, 0x4c, 0xa0, 0x8f, 0xc0, 0xdf, 0xf4, 0x06, 0xe3, 0x25,
0xc1, 0x98, 0x1b, 0xc4, 0x18, 0x76, 0x00, 0x00, 0x00, 0x00, 0x49, 0x45,
0x4e, 0x44, 0xae, 0x42, 0x60, 0x82;


sub NEW
{
	shift->SUPER::NEW(@_[0..2]);
	statusBar();

	$image0 = Qt::Pixmap();
	$image0->loadFromData($image0_data, length ($image0_data), "PNG");
	$image1 = Qt::Pixmap();
	$image1->loadFromData($image1_data, length ($image1_data), "PNG");
	$image2 = Qt::Pixmap();
	$image2->loadFromData($image2_data, length ($image2_data), "PNG");
	$image3 = Qt::Pixmap();
	$image3->loadFromData($image3_data, length ($image3_data), "PNG");
	$image4 = Qt::Pixmap();
	$image4->loadFromData($image4_data, length ($image4_data), "PNG");

	setName("GUI" ) if (name() eq "unnamed");

######## global variables ########################################

	@GUI::listSurfaces=();      #list of strings, where the inputted surfaces are stored 
	push(@GUI::listSurfaces,0); 
	@GUI::listIntersection=();  #list of intersectioncheckboxes
	push(@GUI::listIntersection,intersectallBox);
	@GUI::listDraw=();          #list of drawcheckboxes
	push(@GUI::listDraw,drawallBox);

	@GUI::listColor=();         #list of colorsbuttons
	push(@GUI::listColor,button0);

	@GUI::listLabel=();    #list of labels with number of surfaces
	push(@GUI::listLabel,surfaceLabel);

	@GUI::listTrans=();    #list of sliders for the transparency
	push(@GUI::listTrans,$GUI::slider0);

	@GUI::listChild=(); 	#list of the child processes
	$GUI::counterChild=-1;	#how much cildprocesses exists

	$GUI::counter=0;            #global counter, which gives the number of added curves

	$GUI::stableFlag=0;      #flag, if the stable option is selected or not

	$GUI::counterdraw=0;    #how much surfaces are selected to be drawn
	$GUI::counterintersect=0;  #how much surfaces are selected to be interesected


############################### create the widgets (Buttons, CheckBoxes, etc.) ##########################  

	$GUI::LayoutWidget = Qt::Widget(this, '$GUI::LayoutWidget'); #mainWidget
	setCentralWidget($GUI::LayoutWidget);

	$GUI::LayoutWidget->setSizePolicy( Qt::SizePolicy(&Qt::SizePolicy::Expanding, &Qt::SizePolicy::Expanding)); #so the widgets reorder their positions themselves, if you resize the mainwindow

#main layout
	$GUI::mainlayout = Qt::HBoxLayout($GUI::LayoutWidget);
	$GUI::mainlayout->setSpacing(10);


#layout of the left side
	$GUI::layoutSurfaces = Qt::GridLayout($GUI::mainlayout, '$layoutSurfaces');
#layout of the right side
	$GUI::layoutControl = Qt::GridLayout($GUI::mainlayout,'$layoutControl');


#buttons for selecting all for intersecting and drawing plus labels
	my $selectallLabel = Qt::Label($GUI::LayoutWidget, '$selectallLabel');
	$selectallLabel->setText('Select all:');
	$GUI::layoutSurfaces->addMultiCellWidget($selectallLabel, 1, 1, 1, 1);

	intersectLabel = Qt::Label($GUI::LayoutWidget, "intersectLabel");
	$GUI::layoutSurfaces->addMultiCellWidget(intersectLabel, 0, 0, 5, 7);

	drawLabel = Qt::Label($GUI::LayoutWidget, "drawLabel");
	$GUI::layoutSurfaces->addMultiCellWidget(drawLabel, 0, 0, 2,4);

	$GUI::transLabel = Qt::Label($GUI::LayoutWidget, '$GUI::transLabel');
	$GUI::transLabel->setText('transparency');
	$GUI::layoutSurfaces->addMultiCellWidget($GUI::transLabel, 0, 0, 8,10);

	drawallBox = Qt::CheckBox($GUI::LayoutWidget, "drawallBox");
	drawallBox->setMouseTracking( 1 );
	drawallBox->setChecked(1);
	$GUI::layoutSurfaces->addWidget(drawallBox, 1, 3);

	intersectallBox = Qt::CheckBox($GUI::LayoutWidget, "intersectallBox");
	intersectallBox->setChecked(1);
	$GUI::layoutSurfaces->addWidget(intersectallBox, 1, 6);

	$GUI::transall =Qt::Slider($GUI::LayoutWidget,'$GUI::transall');
	$GUI::transall->setMaxValue(10);
	$GUI::transall->setMinValue(1);
	$GUI::transall->setValue(10);
	$GUI::transall->setTickInterval(1);
	$GUI::transall->setMaximumWidth(80);
	$GUI::transall->setOrientation(&Qt::Horizontal);
	$GUI::layoutSurfaces->addWidget($GUI::transall, 1, 8);

#control complex, with stable, verbose checkboxes,drawbutton and colorchooser intersection
	stableCheckBox = Qt::CheckBox($GUI::LayoutWidget, "stableCheckBox");
	$GUI::layoutControl->addWidget(stableCheckBox, 1, 2);

	$GUI::verboseBox = Qt::CheckBox($GUI::LayoutWidget, '$GUI::verboseBox');
	$GUI::verboseBox ->setText('verbose');
	$GUI::verboseBox->setChecked(1);
	$GUI::layoutControl->addWidget($GUI::verboseBox, 2, 2);

	DrawButton = Qt::PushButton($GUI::LayoutWidget, "DrawButton");
	$GUI::layoutControl->addWidget(DrawButton, 3,0);

# colorchooser for intersection and label
	my $layoutIntersection =Qt::HBoxLayout( -1, '$layoutIntersection'); #own layout for the label and button of the intersectioncolor
	$layoutIntersection->setSpacing(5);

	button0 = Qt::PushButton($GUI::LayoutWidget, "button0");
	my $tmpcolor2=Qt::Color('blue');
	button0->setPaletteBackgroundColor($tmpcolor2);   #the standard color of the intersection ist blue
	button0->setMaximumSize( Qt::Size(15, 15) );
	button0->setName('button0');
	$layoutIntersection->addWidget(button0);

	colerintersectLabel = Qt::Label($GUI::LayoutWidget, "colerintersectLabel");
	colerintersectLabel->setScaledContents( 0 );
	$layoutIntersection->addWidget(colerintersectLabel);
	$GUI::layoutControl->addLayout($layoutIntersection,3,2);

	$GUI::slider0=Qt::Slider($GUI::LayoutWidget,'$GUI::slider0');
	$GUI::slider0->setOrientation(&Qt::Horizontal);
	$GUI::slider0->setMaxValue(9);
	$GUI::slider0->setMinValue(1);
	$GUI::slider0->setTickInterval(1);
	$GUI::slider0->setValue(1);
	$GUI::slider0->setMaximumWidth(80);
	$GUI::layoutControl->addWidget($GUI::slider0, 5, 2);

	$GUI::sliderLabel=Qt::Label($GUI::LayoutWidget,'$GUI::sliderLabel');
	$GUI::sliderLabel->setText('Intersection-<br>transparency');
	$GUI::layoutControl->addWidget($GUI::sliderLabel, 4, 2);

	$GUI::shellText = Qt::TextEdit($GUI::LayoutWidget, '$GUI::shellLine');
	$GUI::shellText->insert('Please be aware, that drawing the intersection will take time! (Depending on your data it may be up to one minute.)');
	$GUI::shellText->setMinimumWidth(400);
	$GUI::shellText->setBaseSize(this->size());
	$GUI::shellText->setReadOnly(1);
	$GUI::shellText->setUndoRedoEnabled(0);
	$GUI::layoutControl->addMultiCellWidget($GUI::shellText,1,6,3,3);

	fileExitAction= Qt::Action(this, "fileExitAction");
	helpManualAction= Qt::Action(this, "helpManualAction");
	helpAboutAction= Qt::Action(this, "helpAboutAction");
	fileClearAction= Qt::Action(this, "fileClearAction");

	addSurface();   #create the first surfaces complex with label, editline to checkboxes and one pushbutton

######## create menu #######

	MenuBar= Qt::MenuBar( this, "MenuBar");

	fileMenu = Qt::PopupMenu( this );
	fileMenu->insertSeparator();
	fileClearAction->addTo( fileMenu );
	fileMenu->insertSeparator();
	fileMenu->insertSeparator();
	fileExitAction->addTo( fileMenu );
	MenuBar->insertItem( "", fileMenu, 1 );

	helpMenu = Qt::PopupMenu( this );
	helpManualAction->addTo( helpMenu );
	helpMenu->insertSeparator();
	helpAboutAction->addTo( helpMenu );
	MenuBar->insertItem( "", helpMenu, 2 );

	languageChange();
	my $resize = Qt::Size(200,90);      #to set the mainwindow on a start size
	$resize = $resize->expandedTo(minimumSizeHint());
	resize( $resize );
	clearWState( &Qt::WState_Polished );

######## connect widgets and slots ###############################

	Qt::Object::connect(helpManualAction, SIGNAL "activated()", this, SLOT "helpManual()");
	Qt::Object::connect(helpAboutAction, SIGNAL "activated()", this, SLOT "helpAbout()");
	Qt::Object::connect(drawallBox, SIGNAL "clicked()", this, SLOT "selectAllDraw()");
	Qt::Object::connect(fileClearAction, SIGNAL "activated()", this, SLOT "clean()");
	Qt::Object::connect(fileExitAction, SIGNAL "activated()", this, SLOT "close()");
	Qt::Object::connect(intersectallBox, SIGNAL "clicked()", this, SLOT "selectAllIntersect()");
	Qt::Object::connect(button0, SIGNAL "clicked()", this, SLOT "changeColor()");
	Qt::Object::connect(DrawButton, SIGNAL "clicked()", this, SLOT "draw()");
	Qt::Object::connect(stableCheckBox, SIGNAL "stateChanged(int)", this, SLOT "setStable()");
	Qt::Object::connect($GUI::verboseBox, SIGNAL "stateChanged(int)", this, SLOT "verbosechanged()");
	Qt::Object::connect($GUI::transall, SIGNAL "sliderReleased()", this, SLOT "transall()");
}

#  Sets the strings of the subwidgets using the current
#  language.

sub languageChange
{
	setCaption(trUtf8("TropicalInsectX") );
	colerintersectLabel->setText( trUtf8("Intersection-<br>color") );
	button0->setText( "" );
	intersectLabel->setText( trUtf8("intersect") );
	drawLabel->setText( trUtf8("draw") );
	drawallBox->setText( "" );
	stableCheckBox->setText( trUtf8("stable") );
	intersectallBox->setText( "" );
	DrawButton->setText( trUtf8("&Draw") );
	DrawButton->setAccel( Qt::KeySequence( trUtf8("Alt+D") ) );
	fileExitAction->setText( trUtf8("Quit") );
	fileExitAction->setMenuText( trUtf8("&Quit") );
	fileExitAction->setAccel( Qt::KeySequence( trUtf8("Ctrl+Q") ) );
	helpManualAction->setText( trUtf8("Manual") );
	helpManualAction->setMenuText( trUtf8("&Manual") );
	helpManualAction->setAccel( Qt::KeySequence( "F1" ) );
	helpAboutAction->setText( trUtf8("About") );
	helpAboutAction->setMenuText( trUtf8("&About") );
	helpAboutAction->setAccel( Qt::KeySequence( "" ) );
	fileClearAction->setMenuText( trUtf8("&Clear") );
	MenuBar->findItem( 1 )->setText( trUtf8("&File") );
	MenuBar->findItem( 2 )->setText( trUtf8("&Help") );
}

############################################# implementation of the slot methods ################################
sub helpManual {
	my $thread = threads->create('helpManual_');
}

sub helpManual_ {
#this slot opens the file manual.pdf with one of the written pdf-viewers, if the user has not installed one of those,
#there appaers a popup-window with the message to open the manual usually
	 my $dirname=File::Basename::dirname(__FILE__);
	 my $error=system("acroread $dirname/manual-tropicalinsect.pdf 1>/dev/null 2>/dev/null || evince $dirname/manual-tropicalinsect.pdf 1>/dev/null 2>/dev/null || kpdf $dirname/manual-tropicalinsect.pdf 1>/dev/null 2>/dev/null || xpdf $dirname/manual-tropicalinsect.pdf 1>/dev/null 2>/dev/null || okular $dirname/manual-tropicalinsect.pdf");
	if($error){
		my $errorWindow = Qt::Dialog(0,'$errorWindow',0,&Qt::WType_Popup);
		my $errorWidget=Qt::Widget($errorWindow, '$errorWidget');
		$errorWidget->setGeometry(Qt::Rect(12,12,300,100));
		my $errorLayout=Qt::GridLayout($errorWidget,'$errorLayout');
		$errorWidget->setSizePolicy( Qt::SizePolicy(&Qt::SizePolicy::Expanding, &Qt::SizePolicy::Expanding));
		my $textLabel = Qt::Label($errorWidget, '$textLabel');
		$textLabel->setScaledContents( 0 );
		$textLabel->setText("It appears you have none of the PDF readers we chose to present the manual.\nPlease open manual-tropicalinsect.pdf, located in the tropicalinsect directory, with a pdf-viewer of your choice.");
		$errorLayout->addWidget($textLabel,0,0);
		my $okButton=Qt::PushButton($errorWidget,'$okButton');
		$okButton->setText('OK');
		Qt::Object::connect($okButton, SIGNAL "clicked()", $errorWindow, SLOT "close()");
		$errorLayout->addWidget($okButton,1,0);  
		$errorWindow->adjustSize();
		$errorWindow->show();
	}
}

sub helpAbout {
#this slot opens a popup-window with information about the licence
	my $aboutWindow = Qt::Dialog(this,'$aboutWindow');
	my $aboutWidget=Qt::Widget($aboutWindow, '$aboutWidget');
	$aboutWindow->setFixedSize(430,300);
	$aboutWidget->setMinimumSize(430,300);

	my $aboutLayout=Qt::GridLayout($aboutWidget,'$aboutLayout');

	my $textLabel = Qt::Label($aboutWidget, '$textLabel');
	$textLabel->setScaledContents( 0 );
	$textLabel->setText("This software was created by Henning Kopp and Nina Bindel, students of the university of Kaiserslautern, in 2010/2011. <br> 
		<br>This program is free software: you can redistribute it and/or modify
		<br>it under the terms of the GNU General Public License as published by   
		<br> the Free Software Foundation, either version 3 of the License, or
		<br>(at your option) any later version.
		<br>     
		<br> This program is distributed in the hope that it will be useful,
		<br> but WITHOUT ANY WARRANTY; without even the implied warranty of
		<br> MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		<br> GNU General Public License for more details.
		<br>                                                                                                                
		<br>You should have received a copy of the GNU General Public License
		<br>along with this program.  If not, see http://www.gnu.org/licenses/. ");
	$aboutLayout->addMultiCellWidget($textLabel,0,0,0,2);

	my $okButton=Qt::PushButton($aboutWidget,'$okButton');
	$okButton->setText('OK');
	$okButton->setMaximumWidth(75);
	Qt::Object::connect($okButton, SIGNAL "clicked()", $aboutWindow, SLOT "close()");
	$aboutLayout->addWidget($okButton,1,1);
	$okButton->show();

	$aboutWindow->show();
}

sub addSurface {
#this slot is activated when the user presses enter after entering his surface
#it creates a new surface complex with label, edit line, one intersectionCheckbox, one drawCheckbox and one coorbutton
	my $pointer = &Qt::Object::sender;  #get the senders reference
	my $scs = 0;
	if(!$pointer){
		$scs = 1;
	}else{
		$GUI::numberEditline = substr($pointer->name() ,8);
#these lines are there to extract the number of the surfaces from the name
#the number is also the place in all five lists, which 
#describe the complex all in all 
		$scs = 1 if ($GUI::numberEditline == $GUI::counter);
	}
	if($scs){
		$GUI::counter++;

		my $Label = Qt::Label($GUI::LayoutWidget,'Label' . $GUI::counter);
		$Label->setText($GUI::counter);
		$GUI::layoutSurfaces->addWidget($Label,1+$GUI::counter,0);
		$Label->show();

		my $linename='EditLine'.$GUI::counter;
		my $EditLine= Qt::LineEdit($GUI::LayoutWidget, $linename);
		$EditLine->setMinimumSize( Qt::Size(255, 0) );
		$EditLine->setMaximumSize( Qt::Size(255, 25) );

		if($GUI::counter==1){
			$EditLine->insert('Please enter a surface and press return. ');
			$EditLine->setSelection(40,40);
			my $palette= $EditLine->palette();
			$palette->setColor(&Qt::ColorGroup::Text, Qt::Color('grey'));
			$EditLine->setPalette($palette);
			Qt::Object::connect($EditLine, SIGNAL "selectionChanged()", this, SLOT "colorText()");
		}
		$EditLine->setFocus() if($GUI::counter!=1);

		Qt::Object::connect($EditLine, SIGNAL "returnPressed()", this, SLOT "addSurface()");                     
		$GUI::layoutSurfaces->addWidget($EditLine,1+$GUI::counter,1);
		$EditLine->show();

# the DrawCheckBox is for selecting a surface to show in the picture, standard status is checked
#a checked DrawCheckBox implies a checked IntersectionCheckBox
		my $DrawCheckBox = Qt::CheckBox($GUI::LayoutWidget, "DrawCheckBox".$GUI::counter);
		$GUI::layoutSurfaces->addWidget($DrawCheckBox,1+$GUI::counter,3);
		$DrawCheckBox->show();

# the IntersectionCheckBox is to select the surfaces to be in the intersection, standard status is checked
		my $IntersectionCheckBox = Qt::CheckBox($GUI::LayoutWidget, "IntersectionCheckBox".$GUI::counter);
		$GUI::layoutSurfaces->addWidget($IntersectionCheckBox,1+$GUI::counter,6);
		$IntersectionCheckBox->show();

#these two connections are to check if all checkboxes of the intersection (resp. drawing) are checked thus 
#implies if the IntersetallCheckBox (resp. DrawallCheckBox) is checked
		Qt::Object::connect($DrawCheckBox, SIGNAL "stateChanged(int)", this, SLOT "changeDraw()");
		Qt::Object::connect($IntersectionCheckBox, SIGNAL "stateChanged(int)", this, SLOT "changeIntersection()");

		my $transSlider= Qt::Slider($GUI::LayoutWidget,"slider".$GUI::counter);
		$transSlider->setOrientation(&Qt::Horizontal);
		$transSlider->setMaxValue(10);
		$transSlider->setMinValue(1);
		$transSlider->setValue(10);
		$GUI::transall->setTickInterval(1);
		$transSlider->setMaximumWidth(80);
		$GUI::layoutSurfaces->addWidget($transSlider,1+$GUI::counter,8);
		$transSlider->show();

		my $colorbutton = Qt::PushButton($GUI::LayoutWidget, "button".$GUI::counter);
		my $tmpcolor=Qt::Color(rand()*255,rand()*255,rand()*255);
		$colorbutton->setPaletteBackgroundColor($tmpcolor);
		$colorbutton->setMaximumSize( Qt::Size(15, 15) );
		$colorbutton->setName('button'.$GUI::counter);
		Qt::Object::connect($colorbutton, SIGNAL "clicked()", this, SLOT "changeColor()");
		$GUI::layoutSurfaces->addWidget($colorbutton,1+$GUI::counter,11);
		$colorbutton->show();

#push the new elements in the special list
		push(@GUI::listIntersection,$IntersectionCheckBox);
		push(@GUI::listDraw,$DrawCheckBox);
		push(@GUI::listColor,$colorbutton);
		push(@GUI::listSurfaces,$EditLine);
		push(@GUI::listLabel,$Label);
		push(@GUI::listTrans,$transSlider);

#to set standard status
		$DrawCheckBox->setChecked(1);
		$IntersectionCheckBox->setChecked(1);

	} # end if
}

sub colorText {
	my $pointer = &Qt::Object::sender;
	my $text=$pointer->text;
	if($text eq 'Please enter a surface and press return. '){
		$pointer->setText('');
		my $black = Qt::Color('black');
		my $palette= $pointer->palette();
		$palette->setColor(&Qt::ColorGroup::Text, $black);
		$pointer->setPalette($palette);
	}
}

sub changeColor
{#this slot is basically the ColorDialog, it is being activated if the user pushes one of the colored pushbuttons
	my $pointer = &Qt::Object::sender;
	my $color = &Qt::ColorDialog::getColor;
	my $name = $pointer->name();
	my $number = substr($name,6);
	$pointer->setPaletteBackgroundColor($color);
	$GUI::listColor[int($number)]=$pointer; 
}

sub selectAllDraw {
#this slot is being activated if the user clicks the selectallDrawBox
#it changes the status of all drawcheckboxes so they have the same status like the
#selectalldrawbox (and implicit the intersectioncheckboxes)
	my $mainCheckBox= &Qt::Object::sender;
	my $status = $mainCheckBox->isChecked();

	$GUI::listDraw[$_]->setChecked($status) foreach(1..$GUI::counter);

	if($status==1){
		$GUI::counterdraw=$GUI::counter;
		intersectallBox->setChecked(1);
	}else{
		$GUI::counterdraw=0;
	}
}

sub selectAllIntersect {
#works analogously to the selectAllDraw
	my $mainCheckBox= &Qt::Object::sender;
	my $status = $mainCheckBox->isChecked();

	$GUI::listIntersection[$_]->setChecked($status) foreach (1..$GUI::counter);
	if($status==1){
		$GUI::counterintersect=$GUI::counter;
	}else{
		$GUI::counterintersect=0;
	}
}

sub transall {
	my $slider=&Qt::Object::sender;
	my $status=$slider->value();

	$GUI::listTrans[$_]->setValue($status) foreach (1..$GUI::counter);
}

sub clean {
#this slot is being activated, when the user activates the "clear" menuitem in the filemenu
#it recreates the first situation of the gui by restarting the program
	this->close();
	system('perl '.__FILE__);
}

sub setStable
{#this slot is being activated when the user change the status of the stablecheckbox
#it only enables or disables the flag stableFlag 
	my $mainCheckBox= &Qt::Object::sender;
	my $status = $mainCheckBox->isChecked();
	$GUI::stableFlag=$status;
}

sub verbosechanged
{
	my $status=&Qt::Object::sender->isChecked();
	if($status){
		$GUI::shellText->show();
		this->setMaximumWidth(3500);
	}
	else{$GUI::shellText->hide();
		this->setMaximumWidth(750);
	}
}

sub sameState {
#this slot is being activated, when the user sets one of the drawcheckboxes status' to 'true', because if a surface should be drawn it has to be in the intersection too,
#otherwise if it is not in the intersection it cannot be drawn
	my $mainCheckBox= &Qt::Object::sender;
	my $status = $mainCheckBox->isChecked();
	my $number = substr($mainCheckBox->name() ,12);
	$GUI::listIntersection[$number]->setChecked($status);
}

sub changeDraw {
#this slot is being activated, if one of the drawcheckboxes is clicked.
#it adapts variables and saves state in list
	my $mainCheckBox= &Qt::Object::sender;
	my $status=$mainCheckBox->isChecked();
	if($status){
		$GUI::counterdraw++;
		my $number = substr($mainCheckBox->name() ,12);
		$GUI::listIntersection[$number]->setChecked(1);
	}else{
		$GUI::counterdraw--;
	}

	drawallBox->setChecked($GUI::counterdraw==$GUI::counter);
}

sub changeIntersection {
#this slot is being activated, if one of the drawcheckboxes is clicked.
#it adapts variables and saves state in list
	my $mainCheckBox= &Qt::Object::sender;
	my $status=$mainCheckBox->isChecked();
	if($status){
		$GUI::counterintersect++;
	}else{
		$GUI::counterintersect--;
		my $number = substr($mainCheckBox->name() ,20);
		$GUI::listDraw[$number]->setChecked(0);
	}
	intersectallBox->setChecked($GUI::counterintersect==$GUI::counter);
}

sub draw
{#this is the "mainslot", it is being activated if the user clicks the Drawbutton.
#it mainly reads the input form the editlines and looks if checkboxes are checked and writes all in one big string
#and starts by a systemcall the program tropicalinsect
	my $stringSurfaces='--surfaces=';
	my $stringDraw='';
	my $stringColor='';
	my $stringTrans='';
	my $stringIntersectionColor='';
	my $stringIntersectionTrans='';
	my ($tmpcolor, $color, $trans, $tmptrans, $i, $j);
	my $error=0;
	my $notintersect=0;
	for($i=1; $i<=$GUI::counter; $i++){
		if(!$GUI::listIntersection[$i]->isChecked()){$notintersect++;}
		elsif($GUI::listIntersection[$i]->isChecked() && $error==0){
			my $tmpedit=$GUI::listSurfaces[$i]->text();
			if($tmpedit ne ''){
#matchen
				unless($tmpedit=~m|[^xyzt0-9^, \-\+\*]|){
					$tmpedit=join('', split(' ',$tmpedit));
					if($i==$GUI::counter){
						$stringSurfaces .= $tmpedit;
					}else{
						$stringSurfaces .= "$tmpedit,";
					}
					if($GUI::listDraw[$i]->isChecked()){
						$j=$i-1;
						if($i==$GUI::counter){
							$stringDraw.=$j-$notintersect;
						}else{
							$j=$j-$notintersect;
							$stringDraw.="$j,";
						}
						$color=$GUI::listColor[$i]->paletteBackgroundColor();
						$tmpcolor='['.$color->red().','.$color->green().','.$color->blue().']';
						$stringColor=$stringColor.$tmpcolor;
						$tmptrans=$GUI::listTrans[$i]->value();
						if($tmptrans==10){
							$trans='1.0';
						}else{
							$trans='0.'.$tmptrans;
						}
						$stringTrans=$stringTrans.'['.$trans.']'.',';
					} #if draw is checked
				}#if unless
				else{
					$error=1;
					my $errorWindow1 = Qt::Dialog(this,'$errorWindow1',0,&Qt::WType_Popup);
					my $errorWidget1=Qt::Widget($errorWindow1, '$errorWidget1');
					$errorWidget1->setGeometry(Qt::Rect(12,12,350,100));
					my $errorLayout1=Qt::GridLayout($errorWidget1,'$errorLayout1');
					$errorWidget1->setSizePolicy( Qt::SizePolicy(&Qt::SizePolicy::Expanding, &Qt::SizePolicy::Expanding));
					my $textLabel1 = Qt::Label($errorWidget1, '$textLabel1');
					$textLabel1->setScaledContents( 0 );
					$textLabel1->setText("An error occurred while trying to draw your surface.<br> Probably there is a typo in your surface ".$tmpedit." Please check the manual for the correct syntax.");
					$errorLayout1->addWidget($textLabel1,0,0);
					my $okButton1=Qt::PushButton($errorWidget1,'$okButton1');
					$okButton1->setText('OK');
					Qt::Object::connect($okButton1, SIGNAL "clicked()", $errorWindow1, SLOT "close()");
					$okButton1->show();
					$errorLayout1->addWidget($okButton1,1,0);
					$errorWindow1->show();  
				}
			}#if edit line contains text  
		}#if intersection is checked     
	}#for


	if($error==0){
		$stringDraw='none' if($GUI::counterdraw==0);
		$stringDraw='' if(drawallBox->isChecked());
		if($stringDraw ne ''){$stringDraw='--show='.$stringDraw;}
		if($stringColor ne ''){$stringColor='--colors='.$stringColor;}
		if($stringTrans ne ''){$stringTrans='--transparencies='.$stringTrans;}


		$color=button0->paletteBackgroundColor();
		$tmpcolor=$color->red().','.$color->green().','.$color->blue();
		$stringIntersectionColor='--intersectioncolor='.$tmpcolor;

		$tmptrans=$GUI::slider0->value();
		if($tmptrans==10){
			$trans='1.0';
		}else{
			$trans='0.'.$tmptrans;
		}
		$stringIntersectionTrans='--intersectiontransp='.$trans;


		my $lastsymbol = substr($stringSurfaces, -1);
		$stringSurfaces=substr($stringSurfaces,0,-1) if($lastsymbol eq ',');

		$lastsymbol = substr($stringDraw,-1);
		$stringDraw=substr($stringDraw,0,-1) if($lastsymbol eq ',');

		$lastsymbol = substr($stringColor,-1);
		$stringColor=substr($stringColor,0,-1) if($lastsymbol eq ':');

		$lastsymbol = substr($stringTrans,-1);
		$stringTrans=substr($stringTrans,0,-1) if($lastsymbol eq ',');

		my $error2;

		#my $env=$ENV{'HOME'};
#$env.='/bin/tropicalinsect';
		#chdir("$env/bin/tropicalinsect");
		#$0='tropicalinsectX.pl';
		my $dirname=File::Basename::dirname(__FILE__);
		my $cmd="polymake --script $dirname/tropicalinsect.pl $stringSurfaces $stringDraw $stringColor $stringIntersectionColor $stringTrans $stringIntersectionTrans ";
		$cmd.="--stable" if($GUI::stableFlag);
		$cmd.=" 2>&1";

		if($GUI::counterChild<50){
			my $thread = threads->create(sub {
					my @outlist;
					open  $GUI::handle,"$cmd |" or die "$cmd could not be executed!\n";
					while(<$GUI::handle>){
						push(@outlist,$_);
						if($GUI::verboseBox->isChecked()){$GUI::shellText->append($_);}
					}                                                                                                                 
					close $GUI::handle;

					if(`echo $?` ne "0\n" ){
						$GUI::errorWindow2 = Qt::Dialog(0,'$GUI::errorWindow2',1,&Qt::WType_Popup);
						my $errorWidget2=Qt::Widget($GUI::errorWindow2, '$errorWidget2');
						$errorWidget2->setGeometry(Qt::Rect(12,12,350,100));
						my $errorLayout2=Qt::GridLayout($errorWidget2,'$errorLayout2');
						$errorWidget2->setSizePolicy( Qt::SizePolicy(&Qt::SizePolicy::Expanding, &Qt::SizePolicy::Expanding));
						my $textLabel2 = Qt::Label($errorWidget2, '$textLabel2');
						$textLabel2->setScaledContents( 0 );
						my $flag = 1;
						my $i=0;
						while($flag && $i<$#outlist){
							if($outlist[$i] eq "polymake:  ERROR: Ambient dimension is unequal to four. Your data cannot be represented\n"){
								$textLabel2->setText("An error occurred while trying to draw your surface.<br> The ambient dimension of the
									intersection is unequal to four. Probably there is a typo in one of your surfaces."); 
								$flag=0;
							}
							elsif($outlist[$i] eq "polymake:  ERROR: mkdir failed: Die Datei existiert bereits\n"){ 
								$textLabel2->setText("An error occurred while trying to draw your surface.<br>There is a tmp directory existent in ~/bin/tropicalinsect/ that should normally have been deleted. Please delete the directory manually.");
								$flag=0;
							}
							elsif($outlist[$i] eq "polymake:  ERROR: Could not create tmp\n"){
								sleep(3);
								draw();
							}
							$i++;
						}
						$errorLayout2->addWidget($textLabel2,0,0);
						my $okButton2=Qt::PushButton($errorWidget2,'$okButton2');
						$okButton2->setText('OK');
						Qt::Object::connect($okButton2, SIGNAL "clicked()", $GUI::errorWindow2, SLOT "close()");
						$okButton2->show();
						$textLabel2->show();
						$errorLayout2->addWidget($okButton2,1,0);
						$GUI::errorWindow2->show();  
					}				
				});
			$thread->detach();
			DrawButton->setDown(1);
			sleep(10);
			DrawButton->setDown(0);
			push(@GUI::listChild,$thread);
			$GUI::counterChild++;
		}
	}
}

############################################ MAIN ########################################

package main;

use Qt;
use GUI;

my $a = Qt::Application(\@ARGV);
my $w = GUI;
$a->setMainWidget($w);
$w->show;
exit $a->exec;
